module Locomotive
  class ContentType

    include Locomotive::Mongoid::Document

    ## extensions ##
    include ::CustomFields::Source
    include Concerns::ContentType::Label
    include Concerns::ContentType::DefaultValues
    include Concerns::ContentType::ItemTemplate
    include Concerns::ContentType::Sync
    include Concerns::ContentType::GroupBy
    include Concerns::ContentType::OrderBy
    include Concerns::ContentType::ClassHelpers

    ## fields ##
    field :name
    field :description
    field :slug
    field :label_field_id,              type: BSON::ObjectId
    field :label_field_name
    field :group_by_field_id,           type: BSON::ObjectId
    field :order_by
    field :order_direction,             default: 'asc'
    field :public_submission_enabled,   type: Boolean, default: false
    field :public_submission_accounts,  type: Array
    field :filter_fields,               type: Array
    field :number_of_entries

    ## associations ##
    belongs_to  :site,      class_name: 'Locomotive::Site', validate: false
    has_many    :entries,   class_name: 'Locomotive::ContentEntry', dependent: :destroy

    ## named scopes ##
    scope :ordered, -> { order_by(updated_at: :desc) }
    scope :by_id_or_slug, ->(id_or_slug) {
      any_of({ _id: id_or_slug }, { slug: id_or_slug })
    }

    ## indexes ##
    index site_id: 1, slug: 1

    ## callbacks ##
    before_validation   :normalize_slug
    before_validation   :sanitize_public_submission_accounts
    before_validation   :sanitize_filter_fields
    after_validation    :bubble_fields_errors_up

    ## validations ##
    validates_presence_of   :site, :name, :slug
    validates_uniqueness_of :slug, scope: :site_id
    validates_size_of       :entries_custom_fields, minimum: 1, message: :too_few_custom_fields

    ## behaviours ##
    custom_fields_for :entries

    ## methods ##

    # Order the list of entries, paginate it if requested
    # and filter it.
    #
    # @param [ Hash ] options Options to filter (where key), order (order_by key) and paginate (page, per_page keys)
    #
    # @return [ Criteria ] A Mongoid criteria if not paginated (array otherwise).
    #
    def ordered_entries(options = nil)
      options ||= {}

      # pagination
      page, per_page = options.delete(:page), options.delete(:per_page)

      # order list
      _order_by_definition = (options || {}).delete(:order_by).try(:split) || self.order_by_definition

      # get list
      _entries = self.entries.order_by([_order_by_definition]).where(options[:where] || {})

      # pagination or full list
      !self.order_manually? && page ? _entries.page(page).per(per_page) : _entries
    end

    # Find a custom field describing an entry based on its id
    # in first or its name if not found.
    #
    # @param [ String ] id_or_name The id of name of the field
    #
    # @return [ Object ] The custom field or nit if not found
    #
    def find_entries_custom_field(id_or_name)
      return nil if id_or_name.nil? # bypass the memoization

      _field = self.entries_custom_fields.find(id_or_name) rescue nil
      _field || self.entries_custom_fields.where(name: id_or_name).first
    end

    protected

    def normalize_slug
      self.slug = self.name.clone if self.slug.blank? && self.name.present?
      self.slug.permalink!(true) if self.slug.present?
    end

    # We do not want to have a blank value in the list of accounts.
    def sanitize_public_submission_accounts
      if self.public_submission_accounts
        self.public_submission_accounts.reject! { |id| id.blank? }
      end
    end

    # We do not want to have a blank value in the list of fields used to filter the entries.
    def sanitize_filter_fields
      if self.filter_fields
        self.filter_fields.reject! { |id| id.blank? }
      end
    end

    def bubble_fields_errors_up
      return if self.errors[:entries_custom_fields].empty?

      hash = { base: self.errors[:entries_custom_fields] }

      self.entries_custom_fields.each do |field|
        next if field.valid?
        key = field.persisted? ? field._id.to_s : field.position.to_i
        hash[key] = field.errors.to_a
      end

      self.errors.set(:entries_custom_fields, hash)
    end

  end
end
